//-< CHANL.CPP >-----------------------------------------------------*--------*
// GOODS                     Version 1.0          (c) 1992  GARRET   *     ?  *
// ( Garret object oriented data storage )                           *   /\|  *
// Virtual active channel realisation                                *  /  \  *
//                           Created:      22-Feb-92   K.A. Knizhnik * / [] \ *
//                           Last update:  24-Oct-92   K.A. Knizhnik * GARRET *
//-------------------------------------------------------------------*--------*

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef __MSDOS__
  #include <io.h>
#elif defined(_WIN32)
  #include <windows.h>
#else
  #include <unistd.h>
#endif

#include "filio.hpp"

global  Dbs_Channel* Dbs_Channel::Manager;

void Io_Error ( void )  {
  perror ( "I/O low level error " );
  abort ();
}

Dbs_Channel::Dbs_Channel ( nat Limit )  {
  ASSERT ( Limit > 0 );
  Active_File_Table = new Dbs_Active_File [ Limit ];
  memset ( Active_File_Table , 0 , Limit * sizeof ( Dbs_Active_File ) );
  Manager = this;
  Timer = 0;
  Active_Files_Number = 0;
  Active_Files_Limit  = Limit;
}

Dbs_Channel::~Dbs_Channel ( void )  {
  for ( int i = Active_Files_Number; i --; )
#ifdef _WIN32
     if ( !CloseHandle ( Active_File_Table[i].File->Handle ) ) Io_Error();
#else
     if ( close ( Active_File_Table[i].File->Handle ) != Ok ) Io_Error();
#endif
     else Active_File_Table[i].File->Handle = INVALID_FILE_HANDLE;
  delete[] Active_File_Table;
}

void Dbs_Channel::Revision ( nat Limit )  {
  ASSERT ( Limit > 0 );
  for ( int i = Active_Files_Number; i --; )  {
     ASSERT ( Active_File_Table[i].Stamp >= 0 );
#ifdef _WIN32
     if ( !CloseHandle ( Active_File_Table[i].File->Handle ) ) Io_Error ();
#else
     if ( close ( Active_File_Table[i].File->Handle ) != Ok ) Io_Error ();
#endif
     else Active_File_Table[i].File->Handle = INVALID_FILE_HANDLE;
  }
  if ( Limit > Active_Files_Limit )  {
     delete[] Active_File_Table;
     Active_File_Table = new Dbs_Active_File [ Limit ];
     memset ( Active_File_Table , 0 , Limit * sizeof ( Dbs_Active_File ) );
  }
  Active_Files_Number = 0;
  Active_Files_Limit  = Limit;
}

void  Dbs_Channel::Erase ( Dbs_Active_File * Del )  {
  *Del = Active_File_Table[--Active_Files_Number];
  Del->File->Channel = Del;
}

large   Dbs_Channel::Scale ( void )  {
  for( int i = Active_Files_Number; i--; Active_File_Table[i].Stamp >>= 1 );
  return Timer = LONG_MAX / 2;
}

int Dbs_Channel::Activate_File ( Dbs_File * File )  {
  int  Mini = Active_Files_Number;
  if ( Active_Files_Number >= Active_Files_Limit )  {
    large  Min_Stamp = large(LONG_MAX) + 1;
    for ( int i = Mini; i --; )
       if ( large(Active_File_Table[i].Stamp) < Min_Stamp )
          Min_Stamp = Active_File_Table[Mini=i].Stamp;
    ASSERT ( Active_File_Table[Mini].Stamp >= 0 );
#ifdef _WIN32
    if ( !CloseHandle ( Active_File_Table[Mini].File->Handle ) ) Io_Error();
#else
    if ( close ( Active_File_Table[Mini].File->Handle ) != Ok ) Io_Error();
#endif
    Active_File_Table[Mini].File->Handle = INVALID_FILE_HANDLE;
  } else Active_Files_Number += 1;

  Active_File_Table[Mini].File = File;
  Active_File_Table[Mini].Stamp = Tick ();
  File->Channel = Active_File_Table + Mini;
#ifdef _WIN32
  File->Handle = CreateFile( File->Name, 
			     (File->Flags & O_RDWR) == O_WRONLY 
			      ? GENERIC_WRITE 
			      : (File->Flags & O_RDWR) == O_RDWR 
			         ? GENERIC_WRITE | GENERIC_READ
			         : GENERIC_READ, 
			     0, NULL, 
			     (File->Flags & O_CREAT) 
			      ? (File->Flags & O_TRUNC) 
			        ? CREATE_ALWAYS
			        : OPEN_ALWAYS
			      : (File->Flags & O_TRUNC) 
			        ? TRUNCATE_EXISTING
			        : OPEN_EXISTING, 
			     (File->Flags & O_SYNC) 
			      ? FILE_FLAG_WRITE_THROUGH|FILE_FLAG_RANDOM_ACCESS
			      : FILE_FLAG_RANDOM_ACCESS, NULL);
  if ( File->Handle == INVALID_HANDLE_VALUE )  Erase ( File->Channel );
  else if (SetFilePointer(File->Handle, File->Curr_Pos, NULL, FILE_BEGIN) 
	   != File->Curr_Pos) 
      Io_Error();
#else
  File->Handle = open( File->Name, File->Flags, File->Mode);
  if ( File->Handle == -1 )  Erase ( File->Channel );
  else if ((large)lseek(File->Handle,File->Curr_Pos,0) != File->Curr_Pos) 
      Io_Error();
#endif
  return (int)File->Handle;
}


