//-< ERMSG.HPP >-----------------------------------------------------*--------*
// Garret Object Class Library                    (c) 1992  GARRET   *     ?  *
//                                                                   *   /\|  *
//  Error handling                                                   *  /  \  *
//                           Created:      16-Mar-92   I.V. Abramow  * / [] \ *
//    Declarations           Last update:  17-Mar-92   I.V. Abramow  * GARRET *
//-------------------------------------------------------------------*--------*

#ifndef __ERMSG_HPP__
    #define __ERMSG_HPP__

    #include <stdarg.h>

    // Error messages.
    //  There is five classes of error messages :
    //   1. Warning -- not very important message, no violation of basic
    //                 principles, but data or user's responce are suspicious,
    //                 or, in the case of programming language translation,
    //                 some nonportable features was used.
    //   2. Error   -- More severe situation occured, but program will continue
    //                 to work ( may be it will only finish the part of work ).
    //   3. Fatal   -- User or hardvare error, program will be terminated
    //   4. Abzac   -- Internal check. Terminate.
    //   5. Trace   -- Debugging message.
    //  Arguments for all of this functions are the same as printf's arguments.

    export          void    Warning( const char* Format, ... );
    export          void    Error  ( const char* Format, ... );
    export  noret   void    Fatal  ( const char* Format, ... );
    export  noret   void    Abzac  ( const char* Format, ... );
    export          void    Trace  ( const char* Format, ... );

    // Output to the stderr could be intercepted by hooked functions.
    // The process of further error message handling depends on
    // the code returned by hook function.

    enum    Err_Hook_Type       {   // Hook functions return type
        Err_Hook_Cont,          // All standard actions should be continued
        Err_Hook_Post,          // Only message postprocessing
        Err_Hook_None           // No actions should be performed
    };

    export  Err_Hook_Type       (*Msg_Warning)(const char* Format,va_list Prm);
    export  Err_Hook_Type       (*Msg_Error)(const char* Format, va_list Prm);
    export  Err_Hook_Type       (*Msg_Fatal)(const char* Format, va_list Prm);
    export  Err_Hook_Type       (*Msg_Abzac)(const char* Format, va_list Prm);
    export  Err_Hook_Type       (*Msg_Trace)(const char* Format, va_list Prm);

    // Global variables which affect the behavior of error message

    export  int     Warn_Lock;  // If != 0 then warning messages are ignored
    export  int     Wrn_Limit;  // If != 0 - maximal number of warning messages
    export  int     Wrn_Count;  // Number of issued warning messages
    export  int     Err_Limit;  // If != 0 - maximal number of error messages
    export  int     Err_Count;  // Number of issued error messages

    // If any of message limits exeeded, then program terminates with code
    // EX_ERRORS

    // Debugging tools

    // L e v e l s   o f  d e b u g

    #define DEBUG_NONE    0           // No debug code at all
    #define DEBUG_CHECK   1           // Some bug catchers
    #define DEBUG_DUMP    2           // Bug catchers and dump routines
    #define DEBUG_TRACE   3           // Procedure execution tracing

    #ifndef DEBUG_LEVEL
    #define DEBUG_LEVEL DEBUG_CHECK
    #endif

    #if DEBUG_LEVEL >= DEBUG_CHECK
        #define ASSERT( x )                                                   \
            do {                                                              \
                if( !(x) )                                                    \
                    Abzac( "ASSERT " #x " IN " __FILE__ " at %d",__LINE__ );  \
            } while(0)
    #else
        #define ASSERT( x )
    #endif

    extern char const*  Sys_Error();

#endif
