//-< FILIO.CPP >-----------------------------------------------------*--------*
// GOODS                     Version 1.0          (c) 1992  GARRET   *     ?  *
// ( Garret object oriented data storage )                           *   /\|  *
// Low level file I/O operations                                     *  /  \  *
//                           Created:      15-Feb-92   K.A. Knizhnik * / [] \ *
//                           Last update:  26-Mar-92   K.A. Knizhnik * GARRET *
//-------------------------------------------------------------------*--------*

#include <string.h>
#include <stdlib.h>
#include "filio.hpp"

#ifdef __MSDOS__
  #include <dos.h>
  #include <io.h>
  #ifdef __386__
     #define         bx           ebx
  #endif
#elif defined(_WIN32)
  #include <windows.h>
  #include <io.h>
#else
  #include <unistd.h>
#endif

Dbs_File::Dbs_File ( const char*  File_Name )  {
  Name = new char [ strlen ( File_Name ) + 1 ];
  strcpy ( Name, File_Name );
  Handle = INVALID_FILE_HANDLE ;
}

Dbs_File::~Dbs_File ( void )  {
  Close ();
  delete[] Name;
}

handle_t Dbs_File::Descriptor ( void )  {
  if ( Handle == INVALID_FILE_HANDLE )  {
     if(Dbs_Channel::Manager->Activate_File(this) < 0) Io_Error();
     return Handle;
  }
  if ( Channel->Stamp >= 0 )
     Channel->Stamp = Dbs_Channel::Manager->Tick ();
  return Handle;
}

handle_t Dbs_File::Force ( void )  {
  if ( Handle == INVALID_FILE_HANDLE )
     if(Dbs_Channel::Manager->Activate_File(this) < 0) Io_Error();
  Channel->Stamp = -1;
  return Handle;
}

void Dbs_File::Free ( void )
  { Channel->Stamp = Dbs_Channel::Manager->Tick (); }

int  Dbs_File::Open  ( int Flags, int Mode )  {
  Close ();
  Curr_Pos = 0;
  this->Mode  = Mode;
  this->Flags = Flags;
  if ( Dbs_Channel::Manager->Activate_File ( this ) == -1 ) return -1;
#ifdef _WIN32
  Last_Pos = GetFileSize( Handle, NULL );
#else
  Last_Pos = lseek ( Handle, 0, 2 );
  lseek ( Handle, 0, 0 );
#endif
  this->Flags &= ~O_TRUNC;
  return (int)Handle;
}

int  Dbs_File::Close   ( void )  {
  if ( Handle != INVALID_FILE_HANDLE )  {
#ifdef _WIN32
     if (!CloseHandle( Handle )) Io_Error ();
     Dbs_Channel::Manager->Erase ( Channel );
     Handle = INVALID_FILE_HANDLE;
#else 
     if ( close ( Handle ) != Ok ) Io_Error ();
     Dbs_Channel::Manager->Erase ( Channel );
     Handle = INVALID_FILE_HANDLE;
#endif
  }
  return Ok;
}

int Dbs_File::Access ( int Mode )  {
#ifdef _WIN32
  return _access ( Name, Mode );
#else
  return access ( Name, Mode );
#endif
}

int  Dbs_File::Unlink ( void )  {
  Close ();
#ifdef _WIN32
  return DeleteFile( Name ) ? Ok : -1;
#else
  return unlink ( Name );
#endif
}

int  Dbs_File::Rename ( const char* New_Name ) {
  Close ();
#ifdef _WIN32
    if (!MoveFile ( Name, New_Name ) ) return -1;
#else 
  #ifdef __MSDOS__
    if ( rename ( Name, New_Name ) ) return -1;
  #else
    if ( link ( Name, New_Name ) || unlink ( Name ) ) return -1;
  #endif
#endif
  if ( strlen ( New_Name ) > strlen ( Name ) )  {
     delete[] Name;
     Name = new char [ strlen ( New_Name ) + 1 ];
  }
  strcpy ( Name, New_Name );
  return Ok;
}

int   Dbs_File::Read  ( void * Buffer, unsigned Quant )  {
#ifdef _WIN32
  DWORD len;
  if (!ReadFile ( Descriptor(), Buffer, Quant, &len, NULL )) { 
    return -1;
  }
  Quant = len;
#else
  Quant = read ( Descriptor(), Buffer, Quant );
#endif
  Curr_Pos += Quant;
  return Quant;
}

int   Dbs_File::Write ( void * Buffer, unsigned Quant )  {
#ifdef _WIN32
  DWORD len;
  if (!WriteFile ( Descriptor(), Buffer, Quant, &len, NULL )) { 
    return -1;
  }
  Quant = len;
#else
  Quant = write ( Descriptor(), Buffer, Quant );
  #ifdef __MSDOS__
    if ( Flags & O_SYNC ) {
       // This DOS function flush file buffers to disk (only for DOS >= 3.3)
       REGS reg;
       reg.h.ah = 0x68;
       reg.x.bx = Handle;
       if( (intdos( &reg, &reg ) & 0xFF00) != 0x6800 ) Io_Error();
    }
  #endif
#endif
  Curr_Pos += Quant;
  if ( Curr_Pos > Last_Pos )  Last_Pos = Curr_Pos;
  return Quant;
}

large Dbs_File::Seek  ( long Offset, int Whence )  {
#ifdef _WIN32
  return Curr_Pos = SetFilePointer( Descriptor(), Offset, NULL, Whence);
#else
  return Curr_Pos = lseek ( Descriptor(), Offset, Whence );
#endif
}

void Dbs_File::Sync ( void )  {
#ifdef _WIN32
  if (!FlushFileBuffers( Handle )) Io_Error();
#else
#ifdef __MSDOS__
    if( Handle != -1 ) {
        REGS reg;
        reg.h.ah = 0x68;
        reg.x.bx = Handle;
        if( (intdos( &reg, &reg ) & 0xFF00) != 0x6800 ) Io_Error();
    }
#else
    if( fsync( Descriptor() ) < 0 ) Io_Error(); 
#endif
#endif
}




