//-< FILIO.HPP >-----------------------------------------------------*--------*
// GOODS                     Version 1.0          (c) 1992  GARRET   *     ?  *
// ( Garret object oriented data storage )                           *   /\|  *
// Low level file I/O operations                                     *  /  \  *
//                           Created:      15-Feb-92   K.A. Knizhnik * / [] \ *
//                           Last update:  26-Mar-92   K.A. Knizhnik * GARRET *
//-------------------------------------------------------------------*--------*

#ifndef __FILIO_HPP__
 #define __FILIO_HPP__

 #include <sys/types.h>
 #include <sys/stat.h>

 #include <fcntl.h>
 #include <stdlib.h>
 
 #include "stdtp.hpp"
 #include "ermsg.hpp"
 #include "chanl.hpp"

 #define  Ok       0

 #if !defined(O_SYNC) && defined(O_FSYNC)
     #define O_SYNC  O_FSYNC
 #endif

 #ifndef O_BINARY
   #define O_BINARY  0
 #endif

 #ifndef O_SYNC
   #define O_SYNC  0x10000  // Synchronous write option
 #endif

 #ifdef _WIN32
   typedef void* handle_t;
   #define INVALID_FILE_HANDLE ((handle_t)-1)
 #else
   typedef int handle_t;
   #define INVALID_FILE_HANDLE -1
 #endif 

 import void Io_Error ( void ) ;

 class Dbs_File {
   private :
    friend  class Dbs_Channel ;
    friend  class Dbs_M_File  ;

   protected :
    Dbs_Active_File * Channel;
    large    Curr_Pos;
    large    Last_Pos;
    handle_t Handle  ;

   public :
    char*   Name    ;
    int     Mode    ;
    int     Flags   ;

    handle_t Descriptor ( void ) ; // return current discriptor
    handle_t Force      ( void ) ; // fix channel in pool

    void    Free        ( void ) ; // take off fixing from channel

    large   Length      ( void ) { return Last_Pos ; }
    large   Tell        ( void ) { return Curr_Pos ; }

    virtual int   Open  ( int Flags, int Mode=0666) ;
    virtual void  Sync  ( void ) ;
    virtual int   Close ( void ) ;
    virtual int   Unlink( void ) ;
    virtual int   Access( int Mode = 0 ) ;
    virtual int   Rename( const char* New_Name ) ;
    virtual int   Read  ( void* Buffer, unsigned Quant ) ;
    virtual int   Write ( void* Buffer, unsigned Quant ) ;
    virtual large Seek  ( long  Offset, int Whence=0   ) ;

    Dbs_File  ( void ) { Handle = INVALID_FILE_HANDLE ; }
    Dbs_File  ( const char* File_Name ) ;
    virtual~Dbs_File ( void ) ;
};

#endif
