//-< PAGES.HPP >-----------------------------------------------------*--------*
// GOODS                     Version 1.0          (c) 1992  GARRET   *     ?  *
// ( Garret object oriented data storage )                           *   /\|  *
// Paging virtual mechanism                                          *  /  \  *
//                           Created:      14-Feb-92   K.A. Knizhnik * / [] \ *
//                           Last update:  16-May-92   K.A. Knizhnik * GARRET *
//-------------------------------------------------------------------*--------*

#ifndef __PAGES_HPP__
 #define __PAGES_HPP__

 #include "stdtp.hpp"
 #include "l2lst.hpp"
 #include "hasht.hpp"

 enum  Pg_Bit_Size  {
   Pg_Tiny  = 9  , // size of page 512   bytes
   Pg_Short = 10 , // size of page 1024  bytes
   Pg_Small = 11 , // size of page 2048  bytes
   Pg_Medium= 12 , // size of page 4096  bytes
   Pg_Large = 14 , // size of page 16384 bytes
   Pg_Huge  = 15   // size of page 32768 bytes
 };

 enum   Page_State { Pg_Clear , Pg_Dirty };

 class  Dbs_Pool;
 class  Dbs_Page;

 class  Pool_Store {
   friend  Dbs_Pool;
   protected :
    Dbs_Pool*    Pool;
    virtual void Save ( Dbs_Page* Page ) = 0;
    virtual void Load ( Dbs_Page* Page ) = 0;
    Pool_Store ( Dbs_Pool& St_Pool ) { Pool = & St_Pool; }
 };

 class  Dbs_Page : public L2_Elem {
   public :
    large        Address;
    Hash_Elem   (Dbs_Page);
    Pool_Store*  Store;
    Page_State   State;
    tiny*        Space;

    ~Dbs_Page() { delete[] Space; }
 };

 class  Dbs_Pool  {
   private:
    static  Dbs_Pool* Chain; // pools may contact
    Dbs_Pool* Next;          // with each other
    Dbs_Page* Pages;         // array of pages
    Dbs_Page* Free;          // chain of free pages
    L2_Elem   Head;          // l2-list to support LRU discipline
    unsigned  Page_Size;     // size of page
    unsigned  Bit_Page_Size; // log2 of page size

    Dbs_Page* Table[ 256 ];  // hash table

   public :
    unsigned  Power ( void )   { return Bit_Page_Size; }
    unsigned  Quant ( void )   { return Page_Size; }

    Dbs_Page* Find  ( Pool_Store* Store, large Address );
    Dbs_Page* New   ( Pool_Store* Store, large Address );

    void      Throw ( Pool_Store* Store );// throw out all pages from this store
    void      Throw ( Dbs_Page* Pg );     // throw out specified page from pool
    void      Throw ( void );             // throw out all pages from pool

    void      Flush ( Pool_Store* Store );// flush all modified pages from store
    void      Flush ( Dbs_Page* Pg );     // flush specified page
    void      Flush ( void );             // flush all modified pages of pool

    Dbs_Pool  ( int Number, Pg_Bit_Size Bit_Size = Pg_Medium );
    ~Dbs_Pool ( void );
 };

#endif
